@@ -11,6 +11,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from .django import django_middleware
-from .request import BaseRequest
-from .response import BaseResponse
+from . import request
+from . import response
+BaseRequest = request.BaseRequest
+BaseResponse = response.BaseResponse
