@@ -15,48 +15,39 @@ from __future__ import annotations
 
 from os import environ
 from re import fullmatch
-from typing import List, Union, Callable, Awaitable, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Awaitable, Callable, List, Union
 
-from .interfaces import RecipeInterface, APIInterface
-from .types import User, FormField, NormalisedFormField, InputFormField
+from ..emailverification.types import User as EmailVerificationUser
+from .interfaces import APIInterface, RecipeInterface
+from .types import InputFormField, NormalisedFormField, User
 
 if TYPE_CHECKING:
     from .recipe import EmailPasswordRecipe
     from supertokens_python.supertokens import AppInfo
-from .constants import (
-    FORM_FIELD_EMAIL_ID,
-    FORM_FIELD_PASSWORD_ID,
-    RESET_PASSWORD
-)
-from supertokens_python.utils import get_filtered_list
+
+from typing import Dict
+
 from httpx import AsyncClient
-from supertokens_python.recipe.emailverification.utils import (
-    InputEmailVerificationConfig,
-    ParentRecipeEmailVerificationConfig,
+from supertokens_python.recipe.emailverification.utils import \
     OverrideConfig as EmailVerificationOverrideConfig
-)
+from supertokens_python.recipe.emailverification.utils import \
+    ParentRecipeEmailVerificationConfig
+from supertokens_python.utils import get_filtered_list
+
+from ..emailverification.utils import ParentRecipeEmailVerificationConfig
+from .constants import (FORM_FIELD_EMAIL_ID, FORM_FIELD_PASSWORD_ID,
+                        RESET_PASSWORD)
 
 
-async def default_validator(_):
+async def default_validator(_: str) -> Union[str, None]:
     return None
 
 
-async def default_handle_post_sign_up(_: User, __: List[FormField]):
-    pass
-
-
-async def default_handle_post_sign_in(_: User):
-    pass
-
-
-async def default_password_validator(value) -> Union[str, None]:
+async def default_password_validator(value: str) -> Union[str, None]:
     # length >= 8 && < 100
     # must have a number and a character
     # as per
     # https://github.com/supertokens/supertokens-auth-react/issues/5#issuecomment-709512438
-    if not isinstance(value, str):
-        return 'Development bug: Please make sure the password field yields a string'
-
     if len(value) < 8:
         return 'Password must contain at least 8 characters, including a number'
 
@@ -72,13 +63,10 @@ async def default_password_validator(value) -> Union[str, None]:
     return None
 
 
-async def default_email_validator(value) -> Union[str, None]:
+async def default_email_validator(value: str) -> Union[str, None]:
     # We check if the email syntax is correct
     # As per https://github.com/supertokens/supertokens-auth-react/issues/5#issuecomment-709512438
     # Regex from https://stackoverflow.com/a/46181/3867175
-    if not isinstance(value, str):
-        return 'Development bug: Please make sure the email field yields a string'
-
     if fullmatch(r'^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,'
                  r'3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$', value) is None:
         return 'Email is not valid'
@@ -87,8 +75,8 @@ async def default_email_validator(value) -> Union[str, None]:
 
 
 def default_get_reset_password_url(
-        app_info: AppInfo) -> Callable[[User], Awaitable[str]]:
-    async def func(_: User):
+        app_info: AppInfo) -> Callable[[User, Dict[str, Any]], Awaitable[str]]:
+    async def func(_: User, __: Dict[str, Any]):
         return app_info.website_domain.get_as_string_dangerous(
         ) + app_info.website_base_path.get_as_string_dangerous() + RESET_PASSWORD
 
@@ -96,8 +84,8 @@ def default_get_reset_password_url(
 
 
 def default_create_and_send_custom_email(
-        app_info: AppInfo) -> Callable[[User, str], Awaitable]:
-    async def func(user: User, password_reset_url_with_token: str):
+        app_info: AppInfo) -> Callable[[User, str, Dict[str, Any]], Awaitable[None]]:
+    async def func(user: User, password_reset_url_with_token: str, _: Dict[str, Any]):
         if ('SUPERTOKENS_ENV' in environ) and (
                 environ['SUPERTOKENS_ENV'] == 'testing'):
             return
@@ -108,8 +96,7 @@ def default_create_and_send_custom_email(
                 'passwordResetURL': password_reset_url_with_token
             }
             async with AsyncClient() as client:
-                await client.post('https://api.supertokens.io/0/st/auth/password/reset', json=data,
-                                  headers={'api-version': '0'})
+                await client.post('https://api.supertokens.io/0/st/auth/password/reset', json=data, headers={'api-version': '0'})  # type: ignore
         except Exception:
             pass
 
@@ -128,26 +115,35 @@ class SignUpFeature:
         self.form_fields = form_fields
 
 
-def normalise_sign_up_form_fields(form_fields: List[InputFormField]) -> List[NormalisedFormField]:
-    normalised_form_fields = []
+def normalise_sign_up_form_fields(
+        form_fields: List[InputFormField]) -> List[NormalisedFormField]:
+    normalised_form_fields: List[NormalisedFormField] = []
     for field in form_fields:
         if field.id == FORM_FIELD_PASSWORD_ID:
             validator = field.validate if field.validate is not None else default_password_validator
-            normalised_form_fields.append(NormalisedFormField(field.id, validator, False))
+            normalised_form_fields.append(
+                NormalisedFormField(
+                    field.id, validator, False))
         elif field.id == FORM_FIELD_EMAIL_ID:
             validator = field.validate if field.validate is not None else default_email_validator
-            normalised_form_fields.append(NormalisedFormField(field.id, validator, False))
+            normalised_form_fields.append(
+                NormalisedFormField(
+                    field.id, validator, False))
         else:
             validator = field.validate if field.validate is not None else default_validator
             optional = field.optional if field.optional is not None else False
-            normalised_form_fields.append(NormalisedFormField(field.id, validator, optional))
-    if len(get_filtered_list(lambda x: x.id == FORM_FIELD_PASSWORD_ID, normalised_form_fields)) == 0:
+            normalised_form_fields.append(
+                NormalisedFormField(
+                    field.id, validator, optional))
+    if len(get_filtered_list(lambda x: x.id ==
+           FORM_FIELD_PASSWORD_ID, normalised_form_fields)) == 0:
         normalised_form_fields.append(
             NormalisedFormField(
                 FORM_FIELD_PASSWORD_ID,
                 default_password_validator,
                 False))
-    if len(get_filtered_list(lambda x: x.id == FORM_FIELD_EMAIL_ID, normalised_form_fields)) == 0:
+    if len(get_filtered_list(lambda x: x.id ==
+           FORM_FIELD_EMAIL_ID, normalised_form_fields)) == 0:
         normalised_form_fields.append(
             NormalisedFormField(
                 FORM_FIELD_EMAIL_ID,
@@ -156,14 +152,6 @@ def normalise_sign_up_form_fields(form_fields: List[InputFormField]) -> List[Nor
     return normalised_form_fields
 
 
-def validate_and_normalise_sign_up_config(config=None) -> SignUpFeature:
-    if config is None:
-        config = {}
-    form_fields = normalise_sign_up_form_fields(
-        config['form_fields'] if 'form_fields' in config else None)
-    return SignUpFeature(form_fields)
-
-
 class SignInFeature:
     def __init__(self, form_fields: List[NormalisedFormField]):
         self.form_fields = form_fields
@@ -174,7 +162,7 @@ def normalise_sign_in_form_fields(
     return list(map(
         lambda y: NormalisedFormField(
             y.id, y.validate if y.id == FORM_FIELD_EMAIL_ID else default_validator, False),
-        get_filtered_list(lambda x: x.id == FORM_FIELD_PASSWORD_ID or x.id == FORM_FIELD_EMAIL_ID, form_fields)))
+        get_filtered_list(lambda x: x.id in (FORM_FIELD_PASSWORD_ID, FORM_FIELD_EMAIL_ID), form_fields)))
 
 
 def validate_and_normalise_sign_in_config(
@@ -185,8 +173,9 @@ def validate_and_normalise_sign_in_config(
 
 class InputResetPasswordUsingTokenFeature:
     def __init__(self,
-                 get_reset_password_url: Union[Callable[[User], Awaitable[str]], None] = None,
-                 create_and_send_custom_email: Union[Callable[[User, str], Awaitable], None] = None):
+                 get_reset_password_url: Union[Callable[[
+                     User, Dict[str, Any]], Awaitable[str]], None] = None,
+                 create_and_send_custom_email: Union[Callable[[User, str, Dict[str, Any]], Awaitable[None]], None] = None):
         self.get_reset_password_url = get_reset_password_url
         self.create_and_send_custom_email = create_and_send_custom_email
 
@@ -195,14 +184,25 @@ class ResetPasswordUsingTokenFeature:
     def __init__(self,
                  form_fields_for_password_reset_form: List[NormalisedFormField],
                  form_fields_for_generate_token_form: List[NormalisedFormField],
-                 get_reset_password_url: Callable[[User], Awaitable[str]],
-                 create_and_send_custom_email: Callable[[User, str], Awaitable]):
+                 get_reset_password_url: Callable[[User, Dict[str, Any]], Awaitable[str]],
+                 create_and_send_custom_email: Callable[[User, str, Dict[str, Any]], Awaitable[None]]):
         self.form_fields_for_password_reset_form = form_fields_for_password_reset_form
         self.form_fields_for_generate_token_form = form_fields_for_generate_token_form
         self.get_reset_password_url = get_reset_password_url
         self.create_and_send_custom_email = create_and_send_custom_email
 
 
+class InputEmailVerificationConfig:
+    def __init__(self,
+                 get_email_verification_url: Union[Callable[[
+                     User, Dict[str, Any]], Awaitable[str]], None] = None,
+                 create_and_send_custom_email: Union[Callable[[
+                     User, str, Dict[str, Any]], Awaitable[None]], None] = None
+                 ):
+        self.get_email_verification_url = get_email_verification_url
+        self.create_and_send_custom_email = create_and_send_custom_email
+
+
 def validate_and_normalise_reset_password_using_token_config(app_info: AppInfo, sign_up_config: InputSignUpFeature,
                                                              config: InputResetPasswordUsingTokenFeature) -> ResetPasswordUsingTokenFeature:
     form_fields_for_password_reset_form = list(map(lambda y: NormalisedFormField(y.id, y.validate, False),
@@ -220,29 +220,33 @@ def validate_and_normalise_reset_password_using_token_config(app_info: AppInfo,
 
 
 def email_verification_create_and_send_custom_email(
-        recipe: EmailPasswordRecipe, create_and_send_custom_email):
-    async def func(user, link):
-        user_info = await recipe.recipe_implementation.get_user_by_id(user.user_id)
+        recipe: EmailPasswordRecipe, create_and_send_custom_email: Callable[[
+            User, str, Dict[str, Any]], Awaitable[None]]) -> Callable[[
+                EmailVerificationUser, str, Dict[str, Any]], Awaitable[None]]:
+    async def func(user: EmailVerificationUser, link: str, user_context: Dict[str, Any]):
+        user_info = await recipe.recipe_implementation.get_user_by_id(user.user_id, user_context)
         if user_info is None:
             raise Exception('Unknown User ID provided')
-        return await create_and_send_custom_email(user_info, link)
+        return await create_and_send_custom_email(user_info, link, user_context)
 
     return func
 
 
 def email_verification_get_email_verification_url(
-        recipe: EmailPasswordRecipe, get_email_verification_url):
-    async def func(user):
-        user_info = await recipe.recipe_implementation.get_user_by_id(user.id)
+        recipe: EmailPasswordRecipe, get_email_verification_url: Callable[[
+            User, Any], Awaitable[str]]) -> Callable[[
+                EmailVerificationUser, Any], Awaitable[str]]:
+    async def func(user: EmailVerificationUser, user_context: Dict[str, Any]):
+        user_info = await recipe.recipe_implementation.get_user_by_id(user.user_id, user_context)
         if user_info is None:
             raise Exception('Unknown User ID provided')
-        return await get_email_verification_url(user_info)
+        return await get_email_verification_url(user_info, user_context)
 
     return func
 
 
 def validate_and_normalise_email_verification_config(
-        recipe: EmailPasswordRecipe, config: Union[InputEmailVerificationConfig, None], override: InputOverrideConfig):
+        recipe: EmailPasswordRecipe, config: Union[InputEmailVerificationConfig, None], override: InputOverrideConfig) -> ParentRecipeEmailVerificationConfig:
     create_and_send_custom_email = None
     get_email_verification_url = None
     if config is None:
@@ -263,7 +267,8 @@ def validate_and_normalise_email_verification_config(
 
 class InputOverrideConfig:
     def __init__(self, functions: Union[Callable[[RecipeInterface], RecipeInterface], None] = None,
-                 apis: Union[Callable[[APIInterface], APIInterface], None] = None,
+                 apis: Union[Callable[[APIInterface],
+                                      APIInterface], None] = None,
                  email_verification_feature: Union[EmailVerificationOverrideConfig, None] = None):
         self.functions = functions
         self.apis = apis
@@ -292,7 +297,8 @@ class EmailPasswordConfig:
 
 
 def validate_and_normalise_user_input(recipe: EmailPasswordRecipe, app_info: AppInfo,
-                                      sign_up_feature: Union[InputSignUpFeature, None] = None,
+                                      sign_up_feature: Union[InputSignUpFeature,
+                                                             None] = None,
                                       reset_password_using_token_feature: Union[
                                           InputResetPasswordUsingTokenFeature, None] = None,
                                       email_verification_feature: Union[InputEmailVerificationConfig, None] = None,
@@ -301,21 +307,23 @@ def validate_and_normalise_user_input(recipe: EmailPasswordRecipe, app_info: App
         override = InputOverrideConfig()
     if reset_password_using_token_feature is None:
         reset_password_using_token_feature = InputResetPasswordUsingTokenFeature()
-    email_verification_feature = validate_and_normalise_email_verification_config(
-        recipe,
-        email_verification_feature,
-        override
-    )
+
     if sign_up_feature is None:
         sign_up_feature = InputSignUpFeature()
     return EmailPasswordConfig(
         SignUpFeature(sign_up_feature.form_fields),
-        SignInFeature(normalise_sign_in_form_fields(sign_up_feature.form_fields)),
+        SignInFeature(
+            normalise_sign_in_form_fields(
+                sign_up_feature.form_fields)),
         validate_and_normalise_reset_password_using_token_config(
             app_info,
             sign_up_feature,
             reset_password_using_token_feature
         ),
-        email_verification_feature,
+        validate_and_normalise_email_verification_config(
+            recipe,
+            email_verification_feature,
+            override
+        ),
         OverrideConfig(functions=override.functions, apis=override.apis)
     )
