@@ -12,10 +12,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import annotations
+
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from supertokens_python.recipe.thirdparty.interfaces import APIOptions, APIInterface
+    from supertokens_python.recipe.thirdparty.interfaces import (APIInterface,
+                                                                 APIOptions)
 
 
 async def handle_apple_redirect_api(api_implementation: APIInterface, api_options: APIOptions):
@@ -27,6 +29,6 @@ async def handle_apple_redirect_api(api_implementation: APIInterface, api_option
     state = body['state'] if 'state' in body else ""
 
     # this will redirect the user...
-    await api_implementation.apple_redirect_handler_post(code, state, api_options)
+    await api_implementation.apple_redirect_handler_post(code, state, api_options, {})
 
     return api_options.response
