@@ -22,6 +22,7 @@ if TYPE_CHECKING:
     from supertokens_python.recipe.thirdparty.provider import Provider
 
 from supertokens_python.exceptions import raise_bad_input_exception
+from supertokens_python.utils import send_200_response
 
 
 async def handle_authorisation_url_api(api_implementation: APIInterface, api_options: APIOptions):
@@ -39,6 +40,4 @@ async def handle_authorisation_url_api(api_implementation: APIInterface, api_opt
         raise_bad_input_exception('The third party provider ' + third_party_id + ' seems to be missing from the backend configs.')
 
     result = await api_implementation.authorisation_url_get(provider, api_options, {})
-    api_options.response.set_json_content(result.to_json())
-
-    return api_options.response
+    return send_200_response(result.to_json(), api_options.response)
